from __future__ import annotations

import argparse
from collections.abc import Iterable
from typing import Sequence

from dochooks import __version__

from ..utils.return_code import FAIL, PASS, ReturnCode
from .check import check
from .pragma import PragmaManager
from .regex import REGEX_CN_WITH_EN, REGEX_EN_WITH_CN


def format(text: str) -> str:
    text = REGEX_CN_WITH_EN.sub(r"\g<cn> \g<en>", text)
    text = REGEX_EN_WITH_CN.sub(r"\g<en> \g<cn>", text)
    return text


def format_lines(lines: Iterable[str]) -> tuple[bool, str, list[tuple[int, str]]]:
    formatted_text: str = ""
    diagnostics: list[tuple[int, str]] = []
    need_format = False
    pragma_manager = PragmaManager()
    for lineno, line in enumerate(lines, 1):
        with pragma_manager.scan(line) as skip_line:
            if skip_line:
                formatted_text += line
                continue
            if not check(line):
                line = format(line)
                need_format = True
                diagnostics.append((lineno, line))
            formatted_text += line
    return need_format, formatted_text, diagnostics


def _format_file(file_path: str) -> ReturnCode:
    with open(file_path, encoding="utf8", newline="\n") as f:
        need_format, formatted_text, diagnostics = format_lines(f)

    if formatted_text:
        with open(file_path, "w", encoding="utf8", newline="\n") as f:
            f.write(formatted_text)
        for lineno, line in diagnostics:
            print(f"Add spaces between EN and CN chars in: {file_path}:{lineno}:\t{line.strip()}")
    return FAIL if need_format else PASS


def main(argv: Sequence[str] | None = None) -> ReturnCode:
    parser = argparse.ArgumentParser(prog="dochooks", description="pre-commit hooks for documentation")
    parser.add_argument("-v", "--version", action="version", version=__version__)
    parser.add_argument("filenames", nargs="*", help="Filenames to check")
    args = parser.parse_args(argv)

    ret_code: ReturnCode = PASS
    for filename in args.filenames:
        ret_code |= _format_file(filename)

    if ret_code != PASS:
        print("")
        print("Snuged en and cn chars have been separated by a space. Now aborting the commit.")
        print('You can check the changes made. Then simply "git add --update ." and re-commit')
        print(
            "If the changes are not correct, you can @SigureMo in that PR, or directly open a issue in https://github.com/ShigureLab/dochooks"
        )
    return ret_code


if __name__ == "__main__":
    raise SystemExit(main())



from __future__ import annotations

import pytest

from dochooks.insert_whitespace_between_cn_and_en_char.check import check_lines
from dochooks.insert_whitespace_between_cn_and_en_char.format import format_lines

NEED_FORMAT: bool = True
NEEDNT_FORMAT: bool = False
cases = [
    (NEEDNT_FORMAT, "中文and英文 # dochooks: skip-line", "中文and英文 # dochooks: skip-line"),
    (
        NEED_FORMAT,
        """
    中文English # 需要格式化
    中文English # dochooks: skip-line
    中文English # 需要格式化
    中文English # dochooks: skip-line
    中文English # 需要格式化
    # dochooks: skip-next-line
    中文English
    中文English # 需要格式化
    # dochooks: skip-next-line
    中文English
    中文English # 需要格式化
    """,
        """
    中文 English # 需要格式化
    中文English # dochooks: skip-line
    中文 English # 需要格式化
    中文English # dochooks: skip-line
    中文 English # 需要格式化
    # dochooks: skip-next-line
    中文English
    中文 English # 需要格式化
    # dochooks: skip-next-line
    中文English
    中文 English # 需要格式化
    """,
    ),
]


@pytest.mark.parametrize("need_format, unformatted, formatted", cases)
def test_check_and_format_lines(need_format: bool, unformatted: str, formatted: str):
    need_format_from_check, _ = check_lines(unformatted.splitlines(keepends=True))
    need_format_from_format, formatted_text, _ = format_lines(unformatted.splitlines(keepends=True))
    assert need_format_from_check == need_format
    assert need_format_from_format == need_format

    assert formatted_text == formatted



from __future__ import annotations

import argparse
from collections.abc import Iterable
from typing import Sequence

from dochooks import __version__

from ..utils.return_code import FAIL, PASS, ReturnCode
from .pragma import PragmaManager
from .regex import REGEX_CN_WITH_EN, REGEX_EN_WITH_CN


def check(string: str) -> bool:
    if REGEX_CN_WITH_EN.search(string) or REGEX_EN_WITH_CN.search(string):
        return False
    return True


def check_lines(lines: Iterable[str]) -> tuple[bool, list[tuple[int, str]]]:
    diagnostics: list[tuple[int, str]] = []
    need_format = False
    pragma_manager = PragmaManager()
    for lineno, line in enumerate(lines, 1):
        with pragma_manager.scan(line) as skip_line:
            if skip_line:
                continue
            if not check(line):
                need_format = True
                diagnostics.append((lineno, line))
    return need_format, diagnostics


def _check_file(file_path: str) -> ReturnCode:
    with open(file_path, encoding="utf8", newline="\n") as f:
        need_format, diagnostics = check_lines(f)
    if need_format:
        for lineno, line in diagnostics:
            print(f"No spaces between EN and CN chars detected at: {file_path}:{lineno}:\t{line.strip()}")
    return FAIL if need_format else PASS


def main(argv: Sequence[str] | None = None) -> ReturnCode:
    parser = argparse.ArgumentParser(prog="dochooks", description="pre-commit hooks for documentation")
    parser.add_argument("-v", "--version", action="version", version=__version__)
    parser.add_argument("filenames", nargs="*", help="Filenames to check")
    args = parser.parse_args(argv)

    ret_code: ReturnCode = PASS
    for filename in args.filenames:
        ret_code |= _check_file(filename)
    return ret_code


if __name__ == "__main__":
    raise SystemExit(main())



from __future__ import annotations

import re

from dochooks.insert_whitespace_between_cn_and_en_char.regex import (
    REGEX_CN_CHAR_STR,
    REGEX_CN_WITH_EN,
    REGEX_EN_CHAR_STR,
    REGEX_EN_WITH_CN,
)


def test_pure_cn_chars_pattern():
    assert re.search(REGEX_CN_CHAR_STR, "纯中文") is not None
    assert re.search(REGEX_EN_CHAR_STR, "纯中文") is None
    assert re.search(REGEX_CN_CHAR_STR, "，") is None
    assert REGEX_CN_WITH_EN.search("纯中文") is None
    assert REGEX_EN_WITH_CN.search("纯中文") is None


def test_pure_en_chars_pattern():
    assert re.search(REGEX_EN_CHAR_STR, "Pure English") is not None
    assert re.search(REGEX_CN_CHAR_STR, "Pure English") is None
    assert REGEX_CN_WITH_EN.search("Pure English") is None
    assert REGEX_EN_WITH_CN.search("Pure English") is None


def test_cn_with_en_pattern():
    assert REGEX_CN_WITH_EN.search("中文后面带English") is not None
    assert REGEX_CN_WITH_EN.sub(r"\g<cn> \g<en>", "中文后面带English") == "中文后面带 English"
    assert REGEX_CN_WITH_EN.sub(r"\g<cn> \g<en>", "中文后面带English啦") == "中文后面带 English啦"


def test_en_with_cn_pattern():
    assert REGEX_EN_WITH_CN.search("English with中文字符") is not None
    assert REGEX_EN_WITH_CN.sub(r"\g<en> \g<cn>", "English with中文字符") == "English with 中文字符"
    assert REGEX_EN_WITH_CN.sub(r"\g<en> \g<cn>", "呀English with 中文字符") == "呀English with 中文字符"


def test_cn_with_punctuation_pattern():
    assert REGEX_CN_WITH_EN.sub(r"\g<cn> \g<en>", "带上标点符号试试，en。") == "带上标点符号试试，en。"
    assert REGEX_EN_WITH_CN.sub(r"\g<en> \g<cn>", "带上标点符号试试，en。") == "带上标点符号试试，en。"



from __future__ import annotations

from contextlib import contextmanager
from enum import Enum


class Pragma(Enum):
    SKIP_NEXT_LINE = "dochooks: skip-next-line"
    SKIP_LINE = "dochooks: skip-line"


class PragmaManager:
    def __init__(self):
        self.skip_line = False

    @contextmanager
    def scan(self, line: str):
        if Pragma.SKIP_LINE.value in line:
            self.skip_line = True
        yield self.skip_line
        self.skip_line = False
        if Pragma.SKIP_NEXT_LINE.value in line:
            self.skip_line = True



from __future__ import annotations

import re
from typing import Final, Pattern

REGEX_CN_CHAR_STR: Final[str] = r"[\u4e00-\u9fa5]"
REGEX_EN_CHAR_STR: Final[str] = r"[a-zA-Z0-9]"

REGEX_CN_WITH_EN: Pattern[str] = re.compile(f"(?P<cn>{REGEX_CN_CHAR_STR})(?P<en>{REGEX_EN_CHAR_STR})")
REGEX_EN_WITH_CN: Pattern[str] = re.compile(f"(?P<en>{REGEX_EN_CHAR_STR})(?P<cn>{REGEX_CN_CHAR_STR})")



from __future__ import annotations

import sys

if sys.version_info >= (3, 9):
    from functools import cache  # type: ignore
else:
    from functools import lru_cache  # type: ignore

    cache = lru_cache(maxsize=None)

__all__ = ["cache"]



from __future__ import annotations

import pytest

from dochooks.insert_whitespace_between_cn_and_en_char.check import check
from dochooks.insert_whitespace_between_cn_and_en_char.format import format

NEED_FORMAT: bool = True
NEEDNT_FORMAT: bool = False

cases: list[tuple[bool, str, str]] = [
    (NEEDNT_FORMAT, "\n", "\n"),
    (NEEDNT_FORMAT, "", ""),
    (NEEDNT_FORMAT, "中文", "中文"),
    (NEED_FORMAT, "中文a", "中文 a"),
    (NEEDNT_FORMAT, "中文\na", "中文\na"),
    (NEED_FORMAT, "中a文", "中 a 文"),
    (
        NEED_FORMAT,
        "这是一行，a line\r\nthe next line呀！\n没了a",
        "这是一行，a line\r\nthe next line 呀！\n没了 a",
    ),
    (
        NEEDNT_FORMAT,
        r"由于没有语法分析，因此并不能将这个与[Python](python.org)分开，公式$\sqrt{\frac{1}{n}}$也是一样",
        r"由于没有语法分析，因此并不能将这个与[Python](python.org)分开，公式$\sqrt{\frac{1}{n}}$也是一样",
    ),
    (
        NEEDNT_FORMAT,
        "不会做额外的事情\r\n比如将在文末插入换行符",
        "不会做额外的事情\r\n比如将在文末插入换行符",
    ),
    (
        NEED_FORMAT,
        """
    这是一段长文本，会混杂一些英文
    啦啦啦，char喵喵喵en啊啦啦啦。xxxx
    en，char。
    """,
        """
    这是一段长文本，会混杂一些英文
    啦啦啦，char 喵喵喵 en 啊啦啦啦。xxxx
    en，char。
    """,
    ),
]


@pytest.mark.parametrize("need_format, unformatted, formatted", cases)
def test_check_and_format(need_format: bool, unformatted: str, formatted: str):
    if need_format:
        assert not check(unformatted)
    else:
        assert unformatted == formatted
        assert check(unformatted)

    assert format(unformatted) == formatted
    # Test format process is stable
    assert check(formatted)
    assert format(formatted) == formatted
